pro stitch_blocks, nblocks=nblocks, root_out=root_out, hdr=hdr, $
    dimx_final=dimx_final, dimy_final=dimy_final, nplanes=nplanes, plane_weight=plane_weight


;;; for observations sliced into several blocks (sub-fields overlapping at the edges):
;;; combine all blocks into a simgle map

;;; for external usage: only the parameters nblocks, root_out and hdr are mandatory
;;; (all individual maps must have been created with the same astrometry as in hdr)

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;;; copyright 2007-2013 H\'el\`ene Roussel
;;; roussel@iap.fr
;;; Institut d'Astrophysique de Paris - Universit\'e Pierre et Marie Curie, France

;;; download and documentation: http://www2.iap.fr/users/roussel/herschel/

;;; Scanamorphos is distributed under the terms of the GNU General Public License
;;; (http://www.gnu.org/licenses/gpl.html).

;;; July 2010: public release version 1

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



if (not keyword_set(dimx_final)) then begin
   dimx_final = sxpar(hdr, 'NAXIS1', /silent)
   dimy_final = sxpar(hdr, 'NAXIS2', /silent)
endif
dimx_final = long(dimx_final)
dimy_final = long(dimy_final)
if (not keyword_set(nplanes)) then nplanes = sxpar(hdr, 'NAXIS3', /silent)
if (not keyword_set(plane_weight)) then plane_weight = 3
plane_error = 1

test_nblocks = sxpar(hdr, 'NBLOCKS', /silent, count=c_nblocks)
if (c_nblocks lt 1 or test_nblocks lt 2) then begin
   sxdelpar, hdr, ['VSCAN', 'NSCANS', 'NBLOCKS', 'BLOCK', 'OPTIONS', 'OPTIONS1', 'OPTIONS2']
   for k = 1, 9 do sxdelpar, hdr, 'OBSID00'+chain(k)
   sxaddpar, hdr, 'NMAPS', nblocks, 'number of stitched maps', after='OBSMODE'
endif

dimxy = dimx_final * dimy_final

cube_combined = fltarr(dimx_final, dimy_final, nplanes)
map_weight_combined = fltarr(dimx_final, dimy_final)
map_error_combined = fltarr(dimx_final, dimy_final)
map_cov = fltarr(dimx_final, dimy_final)
map_sumw = fltarr(dimx_final, dimy_final)


for block = 0L, nblocks - 1 do begin
   cube = readfits(root_out + '_' + chain(block) + '.fits', /silent)
   map_weight = cube(*, *, plane_weight)
   map_error = cube(*, *, plane_error)

   ;;; remove possible bias in signal plane between different blocks:
   tab_off = fltarr(nplanes)
   if block gt 0 then begin
      w_over = where(map_weight * map_weight_combined gt 0.)
      m = min([median(map_weight(w_over)), median(map_weight_combined(w_over))])
      w_over = where(map_weight gt m / 2. and map_weight_combined gt m / 2.)
      m = min([median(map_weight(w_over)), median(map_weight_combined(w_over))])
      w_over = where(map_weight gt 0.7 * m and map_weight_combined gt 0.7 * m $
         and map_error lt 2. * map_error_combined and map_error_combined lt 2. * map_error)
      off = total(cube(w_over) / (map_error(w_over))^2.) / total(1. / (map_error(w_over))^2.) $
         - total(cube_combined(w_over)) / total(map_sumw(w_over))
      print, 'offset of block ' + chain(block) + ' wrt preceding block(s) = ' + chain(off, 5)
      tab_off(0) = off
      if nplanes gt 4 then tab_off(4) = off
   endif
   w_over = where(map_weight gt 0. and map_error gt 0.)

   ;;; add signal plane(s) with offset, weight and error planes with no bias:
   for k = 0, nplanes - 1 do if (k ne plane_weight and k ne plane_error) then $
      cube_combined(w_over+k*dimxy) += ((cube(w_over+k*dimxy) - tab_off(k)) / (map_error(w_over))^2.)
   map_sumw(w_over) += (1. / (map_error(w_over))^2.)
   map_cov(w_over) += float(map_weight(w_over) gt 0.)
   cube_combined(w_over+plane_weight*dimxy) += map_weight(w_over)
   cube_combined(w_over+plane_error*dimxy) += (map_error(w_over))^2.
   map_weight_combined(w_over) = cube_combined(w_over+plane_weight*dimxy) / map_cov(w_over)
   map_error_combined(w_over) = sqrt(cube_combined(w_over+plane_error*dimxy) / map_cov(w_over))
endfor

w = where(map_cov gt 0., complement=wz)
for k = 0, nplanes - 1 do if (k ne plane_weight and k ne plane_error) then $
   cube_combined(w+k*dimxy) /= map_sumw(w)
cube_combined(w+plane_error*dimxy) = sqrt(cube_combined(w+plane_error*dimxy)) / map_cov(w)
for k = 0, nplanes - 1 do if k ne plane_weight then cube_combined(wz+k*dimxy) = !values.f_nan

writefits, root_out + '_combined.fits', cube_combined, hdr


end
